#!/bin/bash
LANG=en_US.utf8

# Thresholds - these may depend on what you do with your computer and what 
# you want it to stay awake for.
CPUTHRESH=5        # in percent 
DISKTHRESH=500     # in blocks/sec (the sar manpage says blocks are 512 bytes in normal cases) 
NETTHRESH=500       # in kB/s
# (no sound playing is ~25 while playing mp3 w/ pcm+master turned all the way to zero is ~250)

# Sample times (in seconds) - again needs may vary
CPUTIME=1
DISKTIME=2
NETTIME=2

CURRENTTIME=`date "+%H"`;
if [ $CURRENTTIME -ge 8 ];then
     if [ $CURRENTTIME -lt 22 ]; then
	echo "onlinetime - no shutdown";
	exit;
     fi
fi

# check if a torrent is need to download
ACTIVE_TORRENT=`/usr/sbin/munin-run rtom_vol|awk '/incomplete/ {printf "%d\n",$2}'`
if [ -n $ACTIVE_TORRENT ];then
    if [ $ACTIVE_TORRENT -gt 0 ];then
        echo $ACTIVE_TORRENT "Active Torrent";
        exit;
    fi
fi

ACTIVE_CLIENTS=`/usr/sbin/arp -an|/bin/grep -cE "\(192\.168\.99\.[[:digit:]]+\) at ([abcdef0123456789]{2}:{0,1}){6}"`
if [ $ACTIVE_CLIENTS -gt 0 ];then
	echo $ACTIVE_CLIENTS "clients online";
	exit;
fi

ping="ping -c 1"
for entry in $(/usr/sbin/arp -an|/usr/bin/awk '/\(192\.168\.99\.[[:digit:]]+\)/ {gsub(/[\(\)]/,"",$2);printf("%s\n",$2)}'); 
do
  ${ping} ${entry} > /dev/null
  if [ $? -eq 0 ];then
    echo "$entry is active";
    exit;
  else
    echo "$entry is offline";
  fi
done

# check raid status / don't interupt validate/rebuild
RAIDSTATUS=`/usr/sbin/munin-run raid_status|awk '{SUM += $2} END { printf ("%d\n",SUM/NR)}'`
if [ -n $RAIDSTATUS ];then
    if [ $RAIDSTATUS -lt 2 ];then
	echo "RAID Status";
        exit;
    fi
fi

BBUSTATUS=`/usr/sbin/munin-run raid_bbu_c0|awk '/bbu_ready.value/ {SUM += $2} END { printf ("%d\n",SUM)}'`;
if [ -n $BBUSTATUS ]; then
    if [ $BBUSTATUS -lt 190 ];then
	echo "BBU Status";
	exit;
    fi
fi

# Check CPU usage defined as non-idle time.
CPU=`/usr/bin/sar $CPUTIME 5 | /usr/bin/awk '/Average/ { printf "%d\n",100-$8}'`
if [ -n $CPU ];then
    if [ $CPUTHRESH -lt $CPU ];then
	echo "CPU Time " $CPU;
    	exit;
    fi
fi

# Check disk usage (read + write)
DISK=`/usr/bin/sar -b $DISKTIME 5 | /usr/bin/awk '/Average/ { printf "%d\n",$5 + $6 }'`

if [ -n $DISK ];then
    if [ $DISKTHRESH -lt $DISK ];then
	echo "DISK Time";
        exit;
    fi
fi

# Check net usage (exclude wifi0 which appears to have activity all the time for some weird reason)
NET=`/usr/bin/sar -n DEV $NETTIME 5 | /usr/bin/awk '/Average/ { SUM += $5 + $6 } END { printf ("%d\n",SUM)}'`
if [ $NETTHRESH -lt $NET ]; then
    echo "NET Time";
    exit;
fi

# check for remote login
REMOTELOGIN=`last | head | grep "pts/.*still logged in"`
if [ -n "$REMOTELOGIN" ];then
    echo "REMOTELOGIN";
    exit;
fi

if [ -e /tmp/suspend.last ];then
 if [ $(($(date +%s) - $(date -r /tmp/suspend.last +%s))) -lt 600 ]; then 
  echo "suspend.last";
  exit
 fi
fi

if [ -e /var/run/munin/munin-update.lock];then
    echo "munin lock";
    exit
fi

# check if a suspend is ongoing
if [ -e /tmp/suspend.lock ]; then
 exit
fi

# Go into standby
touch /tmp/suspend.lock

BASEDATE=`date "+%s" -d "00:00:00"`;
if [ $CURRENTTIME -ge 10 ];then
   WAKEUP=`expr $BASEDATE + 122400`;
   else
   WAKEUP=`expr $BASEDATE + 36000`;
fi

echo $WAKEUP > /sys/class/rtc/rtc0/wakealarm
echo mem > /sys/power/state

rm /tmp/suspend.lock
touch /tmp/suspend.last

/sbin/start -q anacron
